(****************************************************************)
(* Application example with AVRco LCD Graphics Library using    *)
(* the Philips PCF8548 LCD Controller. This chip has an I2C     *)
(* interface and is built into the Bolymin LCD Graphic Display  *)
(* BF9864A. The PCF chip has no busy output, no read-back of    *)
(* its frame memory and only supports the column mode.          *)
(*                                                              *)
(* The resolution of the display is 98x64 pixels. Our driver    *)
(* supports only multiples of 8 in x and y direction. So we set *)
(* the drivers resolution to 104x64 pixels, having in mind that *)
(* the last 6 x-pixels are not displayed.                       *)
(****************************************************************)

program Graph_BO9864;

{$NOSHADOW}
{ $W+ Warnings}{Warnings off}

Device = XMega256A3U, VCC = 3.3;

Import SysTick,
       SerPortC0,
       TWI_E,
       LCDGraphic,
       BeepPort;

From System Import;
From LCDGraphic Import CharSet;     {block CharSet, pixels}

Define
  // The XMegas don't provide any Oscillator fuses.
  // So the application must setup the desired values
  // possible OSC types: extXTAL, extClock, ext32kHz, int32Khz, int2MHz, int32MHz
  //>> CPU=32MHz, PeripherX4=32MHz, PeripherX2=32MHz
  OSCtype           = int32MHz,
      PLLmul        = 4,
      prescB        = 1,
      prescC        = 1,
      overdrive;
  SysTick           = 10;              {msec}
  StackSize         = $0060, iData;
  FrameSize         = $0080, iData;
  SerPortC0         = 19200, Stop2;     {Baud, StopBits|Parity}
  RxBufferC0        = 8, iData;
  TxBufferC0        = 8, iData;
  TWIprescE         = TWI_BR100;
  LCDGraphic        = 104, 64, 8;      {x-pix, y-pix, accesswidth}
  // use the Column mode controller PCF8548 with I2C/TWI interface
  LCDgraphMode      = column, iData;      // column mode
  DefCharSet        = 'Graphchars.pchr';  {FileName}
  GViewports        = 1, iData;            {logical ViewPorts, scalings}
  TGraphStr         = 16;                  {Graphic Text String Length}
  BeepPort          = PortR.0;

Implementation

{$IDATA}

const
  // definitions for 2 bitmap types
  BmpSize8x8     : byte    = (8*8 div 8) + 2;
  BmpSize16x32   : byte    = (16*32 div 8) + 2;

{--------------------------------------------------------------}
{ Type Declarations }
type
  // we need a few array types where we can insert the bitmaps
  tBMP8          = array[1..BmpSize8x8] of byte;
  tArrows        = array[0..3, 1..BmpSize8x8] of byte;
  tChips         = array[0..15, 1..BmpSize16x32] of byte;

{--------------------------------------------------------------}
const
  // PCF8548 definitions
  PCFaddr        : byte    = $3C;                // TWI address
  PCFctrl        : byte    = $00;                // control type command byte
  PCFdata        : byte    = $40;                // data type command byte

  // constant which hold contain the bitmaps
  Arrows         : tArrows = 'Arr_N.pbmp', 'Arr_W.pbmp', 'Arr_S.pbmp', 'Arr_E.pbmp';
  Folder8        : tBMP8   = 'Folder8.pbmp';

  ChipsV         : tChips  = 'ChipV0.pbmp', 'ChipV1.pbmp', 'ChipV2.pbmp', 'ChipV3.pbmp',
                 'ChipV4.pbmp', 'ChipV5.pbmp', 'ChipV6.pbmp', 'ChipV7.pbmp',
                 'ChipV8.pbmp', 'ChipV9.pbmp', 'ChipV10.pbmp', 'ChipV11.pbmp',
                 'ChipV12.pbmp', 'ChipV13.pbmp', 'ChipV14.pbmp', 'ChipV15.pbmp';

  ChipsH         : tChips  = 'ChipH0.pbmp', 'ChipH1.pbmp', 'ChipH2.pbmp', 'ChipH3.pbmp',
                 'ChipH4.pbmp', 'ChipH5.pbmp', 'ChipH6.pbmp', 'ChipH7.pbmp',
                 'ChipH8.pbmp', 'ChipH9.pbmp', 'ChipH10.pbmp', 'ChipH11.pbmp',
                 'ChipH12.pbmp', 'ChipH13.pbmp', 'ChipH14.pbmp', 'ChipH15.pbmp';

{ $define BF9864}

{$ifdef BF9864}
  // PCF8548 should be initialized in one access,
  // so we use an array which will be written at once
  // into the controller
  PCFiniParms    : array[0..8] of byte =
                 ($29,  // FunctionSet register, ext reg on
                 // v   = 0 horizontal addressing
                 // pd  = 0 set active
                 // mx  = 0 normal x addressing
                 // my  = 1 mirror y addressing
                 $0D,   // Display config, LSB top,
                 $12,   // Bias system register
                 // bs[2:0] = 010 1:65 mux rate
                 $BD,   // VOP register set Vlcd to 8.6V
                 // min. $B0, max. $BF -> contrast ($BD)
                 $05,   // Temperatur control reg,

                 $28,  // FunctionSet register, ext reg off
                 // v   = 0 horizontal addressing
                 // pd  = 0 set active
                 // mx  = 0 normal x addressing
                 // my  = 1 mirror y addressing
                 $0C,  // Display control register, normal mode, on
                 $11,  // HV-gen 3x
                 $05); // Vlcd range = high
{$else}
  PCFiniParms    : array[0..6] of byte =
                    ($21,   // FunctionSet register, ext reg on
                            // v   = 0 horizontal addressing
                            // pd  = 0 set active
                            // mx  = 0 normal x addressing
                            // my  = 0 mirror y addressing
                     $12,   // Bias system register
                            // bs[2:0] = 010 1:65 mux rate
                     $0C,   // Display config, LSB top,
                     $BD,   // VOP register set Vlcd to 8.6V
                            // min. $B0, max. $BF -> contrast ($BD)
                     $28,   // FunctionSet register, ext reg off
                            // v   = 0 horizontal addressing
                            // pd  = 0 set active
                            // mx  = 0 normal x addressing
                            // my  = 1 mirror y addressing
                     $0C,   // Display control register, normal mode, on
                     $05);  // Vlcd range = high

{$endif}

{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
var
  LCDres[@PortF, 0] : bit;
  bb, idx           : byte;
  i                 : integer;

{--------------------------------------------------------------}
{ functions }
Procedure InitPorts;
begin
  // The PCF8548 always needs at least two reset pulses
  DDRF.0:= 1;                          // LCD reset pin
  LCDres:= 1;
  mDelay(1);
  LCDres:= 0;
  mDelay(1);
  LCDres:= 1;
  mDelay(1);
  LCDres:= 0;
  mDelay(1);
  LCDres:= 1;
  mDelay(1);
end;

Procedure GraphInit;
begin
  mDelay(100);
  // write init values to controller
  TWIoutE(PCFaddr, PCFctrl, PCFiniParms);
  mDelay(1000);
  // clear entire display memory
  TWIoutE(PCFaddr, PCFctrl, $80);       // x-addr 0
  TWIoutE(PCFaddr, PCFctrl, $40);       // y-addr 0
  for i:= 0 to (102*9) do
    TWIoutE(PCFaddr, PCFdata, 0);
  endfor;
  gclrscr(0);
  gdisprefresh;
end;

{$D-}
UserDevice GraphIOS(cmd : byte; arg : byte);
begin
(* Display Refresh with "GraphIOS" *)
(* cmd = 0   arg = rowaddr    set rowaddr to arg *)
(* cmd = 1   arg = pixels     write data byte    *)
  if cmd = 0 then
    // set row addr
    TWIoutE(PCFaddr, PCFctrl, $80);       // x-addr <- 0
    // set column address, $40 is the y-addr command
    arg:= arg + $40;
    TWIoutE(PCFaddr, PCFctrl, arg);       // y-addr <- arg
  else
    // data out
    TWIoutE(PCFaddr, PCFdata, arg);
  endif;
end;
{$D+}

{--------------------------------------------------------------}
{ Main Program }
{$IDATA}
begin
  InitPorts;
  EnableInts($87);
  if TWIstatE(PCFaddr) then
    BeepChirpH(2);
  else
    BeepSiren(0, 2);
  endif;

  TWIinpE(PCFaddr, bb);               // status reg
  Write(SerOutC0, ByteToHex(bb)+' ');
  
  GraphInit;

  TWIinpE(PCFaddr, bb);               // status reg
  WriteLn(SerOutC0, ByteToHex(bb));

  gDrawRect(20, 6, 94, 56, $FF);
  gDrawRect(0, 1, 97, 63, $AA);
  gdisprefresh;
  mDelay(1000);
  gSetTextJustify(alHorCenter, alVertBottom);
  gSetTextBkGnd(bkTransp);
  gSetTextMode(wmSetPix);
  gDrawString(35, 53, 1, 1, TxtRot0, 'Test');
  gSetTextMode(wmXorPix);
  gDrawString(60, 33, 2, 2, TxtRot0, 'E-LAB');
  gDrawString(60, 42, 1, 1, TxtRot0, 'Computers');
  gDrawString(31, 27, 1, 1, TxtRot90, 'AVRco');
  gdisprefresh;
  gDrawBitMap(2, 2, @Folder8, wmSetPix);
  gDrawBitMapN(6, 42, @Arrows[0, 1], wmSetPix);
  gdisprefresh;
  mDelay(10);

  for i:= 3 to 48 do
    gDrawBitMap(2, i - 1, @Folder8, wmclrpix);
    gDrawString(32 + i, 53, 1, 1, TxtRot0, 'Test');
    gDrawBitMap(2, i, @Folder8, wmSetPix);
    gDrawString(33 + i, 53, 1, 1, TxtRot0, 'Test');
    gdisprefresh;
    mDelay(100);
  endfor;
  for i:= 47 downto 2 do
    gDrawBitMap(2, i + 1, @Folder8, wmclrpix);
    gDrawString(34 + i, 53, 1, 1, TxtRot0, 'Test');
    gDrawBitMap(2, i, @Folder8, wmSetPix);
    gDrawString(33 + i, 53, 1, 1, TxtRot0, 'Test');
    gdisprefresh;
    mDelay(100);
  endfor;
  gDrawBitMap(2, 2, @Folder8, wmclrpix);
  gDrawBitMap(6, 52, @Folder8, wmSetPix);
  gdisprefresh;

  loop
    inc(bb);
    if bb > 3 then
      bb:= 0;
    endif;
    gDrawBitMapN(6, 42, @Arrows[bb, 1], wmSetPix);

    for idx:= 0 to 15 do
      gDrawBitMapN(2, 8, @ChipsV[idx, 1], wmSetPix);
      gDrawBitMapN(60, 45, @ChipsH[idx, 1], wmSetPix);
      gdisprefresh;
      mDelay(100);
    endfor;
    mDelay(500);

    inc(bb);
    if bb > 3 then
      bb:= 0;
    endif;
    gDrawBitMapN(6, 42, @Arrows[bb, 1], wmSetPix);

    for idx:= 15 downto 0 do
      gDrawBitMapN(2, 8, @ChipsV[idx, 1], wmSetPix);
      gDrawBitMapN(60, 45, @ChipsH[idx, 1], wmSetPix);
      gdisprefresh;
      mDelay(100);
    endfor;
    mDelay(500);
  endloop;
end Graph_BO9864.

